from argparse import ArgumentParser
from pickle import load, dump
from os.path import join, exists
from os import mkdir
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import RandomForestClassifier
from sklearn import metrics
import seaborn as sn
from scapy.all import *

from time import time

table = {
    'size': [],
    'tcp_sport': [], 'tcp_dport': [], 'tcp_flags': [],
    'udp_sport': [], 'udp_dport': [],
    'inter_time': [],
    'feature': []
}

def is_device_new(mac): #!
    query = join('devices', lower_mac(mac))
    return not exists(query)
    
def is_feature_new(mac, feature): #!
    query = join('devices', lower_mac(mac), feature)
    return not exists(query)

def lower_mac(mac): #!
    return mac.lower().replace(':', '.')

def get_target_path(mac, feature): #!
    target_path = join('devices', lower_mac(mac), feature)
    if is_device_new(mac): mkdir(join('devices', lower_mac(mac)))
    if is_feature_new(mac, feature): mkdir(target_path)
    return target_path
    
def get_training_set(mac, feature):
    feature_path = get_target_path(mac, feature)
    return load(open(join(feature_path, 'training.set'), 'rb'))
    
def get_rf_table(mac, training_set): # mod!
    global table
    rf_table = table
    for i, (timestamp, packet, is_feature) in enumerate(training_set):
        p = packet/IP()
        # print(p.show())
        # print(is_feature)
        size = len(p)
        feature = int(is_feature) 
        tcp_sport = 0
        tcp_dport = 0
        tcp_flags = 0
        if TCP in p:
            tcp_sport = p[TCP].sport
            tcp_dport = p[TCP].dport
            tcp_flags = int(hash(p[TCP].flags))
        udp_sport = 0
        udp_dport = 0
        if UDP in p:
            udp_sport = p[UDP].sport
            udp_dport = p[UDP].dport
        inter_time = 0
        if i > 0:
            inter_time = int((p.time - training_set[i - 1][1].time) * 10000)
        rf_table['size'].append(size)
        rf_table['feature'].append(feature)
        rf_table['tcp_sport'].append(tcp_sport)
        rf_table['tcp_dport'].append(tcp_dport)
        rf_table['tcp_flags'].append(tcp_flags)
        rf_table['udp_sport'].append(udp_sport)
        rf_table['udp_dport'].append(udp_dport)
        rf_table['inter_time'].append(inter_time)
    return rf_table
    
def get_train_test_split(rf_table):
    global table
    c = list(table.keys())
    #pd.set_option('display.max_rows', None)
    data_frame = pd.DataFrame(rf_table, columns=c)
    print(data_frame)
    x = data_frame[c[:-1]]
    y = data_frame[c[-1]]
    return train_test_split(x, y, test_size=0.25, random_state=0)

def get_training_model(rf_table):
    x_train, x_test, y_train, y_test = get_train_test_split(rf_table)
    model = RandomForestClassifier(n_estimators=100)
    model.fit(x_train, y_train)
    return model
    
def get_model_accuracy(rf_table, model):
    _, x_test, _, y_test = get_train_test_split(rf_table)
    y_pred = model.predict(x_test)
    return metrics.accuracy_score(y_test, y_pred)
    
def make_prediction_against_model(model, list):
    return model.predict([list])
    
def main(): #!
    # Handle arguments
    argument_parser = ArgumentParser()
    argument_parser.add_argument('--mac', required=True)
    argument_parser.add_argument('--feature', required=True)
    arguments = argument_parser.parse_args()
    
    training_set = get_training_set(arguments.mac, arguments.feature)
    rf_table = get_rf_table(arguments.mac, training_set)
    
    model = get_training_model(rf_table)
    model_accuracy = get_model_accuracy(rf_table, model)
    
    pickle_dump = (model, model_accuracy, arguments.mac, arguments.feature)
    pickle_path = join(get_target_path(arguments.mac, arguments.feature), 'training.model')
    dump(pickle_dump, open(pickle_path, 'wb'))
    
    print(model_accuracy)
    #start = time()
    #print(make_prediction_against_model(model, [136, 1]))
    #end = time()
    #print(end - start)
    
main()
# sudo python3 create_training_model.py --mac F4:B8:5E:5F:A9:57 --feature live_stream
